--- old/usr/src/cmd/sgs/tools/common/leb128.c	2012-10-11 22:38:20.423160716 -0400
+++ new/usr/src/cmd/sgs/tools/common/leb128.c	2012-10-11 22:38:20.434487672 -0400
@@ -195,7 +195,8 @@
  */
 uint64_t
 dwarf_ehe_extract(unsigned char *data, uint64_t *dotp, uint_t ehe_flags,
-    unsigned char *eident, uint64_t sh_base, uint64_t sh_offset)
+    unsigned char *eident, boolean_t frame_hdr, uint64_t sh_base,
+    uint64_t sh_offset, uint64_t dbase)
 {
 	uint64_t    dot = *dotp;
 	uint_t	    lsb;
@@ -281,17 +282,27 @@
 	/*
 	 * If value is relative to a base address, adjust it
 	 */
-	if (result) {
-		switch (ehe_flags & 0xf0) {
-		case DW_EH_PE_pcrel:
-			result += sh_base + sh_offset;
-			break;
+	switch (ehe_flags & 0xf0) {
+	case DW_EH_PE_pcrel:
+		result += sh_base + sh_offset;
+		break;
 
-		case DW_EH_PE_datarel:
+	/*
+	 * datarel is relative to .eh_frame_hdr if within .eh_frame,
+	 * but GOT if not.
+	 */
+	case DW_EH_PE_datarel:
+		if (frame_hdr)
 			result += sh_base;
-			break;
-		}
+		else
+			result += dbase;
+		break;
 	}
+
+	/* Truncate the result to its specified size */
+	result = (result << ((sizeof (uint64_t) - fsize) * 8)) >>
+	    ((sizeof (uint64_t) - fsize) * 8);
+
 	*dotp = dot;
 	return (result);
 }
